﻿using System;

namespace EonaCat.Quic.Helpers;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

public class ByteArray
{
    private readonly byte[] _array;
    private readonly int _length;

    private int _offset;

    public ByteArray(byte[] array)
    {
        _array = array;
        _length = array.Length;
        _offset = 0;
    }

    public byte ReadByte()
    {
        var result = _array[_offset++];
        return result;
    }

    public byte PeekByte()
    {
        var result = _array[_offset];
        return result;
    }

    public byte[] ReadBytes(int count)
    {
        var bytes = new byte[count];
        Buffer.BlockCopy(_array, _offset, bytes, 0, count);

        _offset += count;

        return bytes;
    }

    public byte[] ReadBytes(IntegerVar count)
    {
        return ReadBytes(count.Value);
    }

    public ushort ReadUInt16()
    {
        var bytes = ReadBytes(2);
        var result = ByteHelpers.ToUInt16(bytes);

        return result;
    }

    public uint ReadUInt32()
    {
        var bytes = ReadBytes(4);
        var result = ByteHelpers.ToUInt32(bytes);

        return result;
    }

    public IntegerVar ReadIntegerVar()
    {
        // Set Token Length and Token
        var initial = PeekByte();
        var size = IntegerVar.Size(initial);

        var bytes = new byte[size];
        Buffer.BlockCopy(_array, _offset, bytes, 0, size);
        _offset += size;

        return bytes;
    }

    public IntegerParts ReadGranularInteger(int size)
    {
        var data = ReadBytes(size);
        IntegerParts result = data;

        return result;
    }

    public StreamId ReadStreamId()
    {
        var streamId = ReadBytes(8);
        StreamId result = streamId;

        return result;
    }

    public bool HasData()
    {
        return _offset < _length;
    }
}