﻿using System;

namespace EonaCat.Quic.Helpers;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

public class IntegerParts
{
    public const ulong MaxValue = 18446744073709551615;

    public IntegerParts(ulong integer)
    {
        Value = integer;
    }

    public ulong Value { get; }

    public byte Size => RequiredBytes(Value);

    public byte[] ToByteArray()
    {
        return Encode(Value);
    }

    public static implicit operator byte[](IntegerParts integer)
    {
        return Encode(integer.Value);
    }

    public static implicit operator IntegerParts(byte[] bytes)
    {
        return new IntegerParts(Decode(bytes));
    }

    public static implicit operator IntegerParts(ulong integer)
    {
        return new IntegerParts(integer);
    }

    public static implicit operator ulong(IntegerParts integer)
    {
        return integer.Value;
    }

    public static byte[] Encode(ulong integer)
    {
        var requiredBytes = RequiredBytes(integer);
        var offset = 8 - requiredBytes;

        var uInt64Bytes = ByteHelpers.GetBytes(integer);

        var result = new byte[requiredBytes];
        Buffer.BlockCopy(uInt64Bytes, offset, result, 0, requiredBytes);

        return result;
    }

    public static ulong Decode(byte[] bytes)
    {
        var i = 8 - bytes.Length;
        var buffer = new byte[8];

        Buffer.BlockCopy(bytes, 0, buffer, i, bytes.Length);

        var res = ByteHelpers.ToUInt64(buffer);

        return res;
    }

    private static byte RequiredBytes(ulong integer)
    {
        byte result = 0;

        if (integer <= byte.MaxValue) /* 255 */
        {
            result = 1;
        }
        else if (integer <= ushort.MaxValue) /* 65535 */
        {
            result = 2;
        }
        else if (integer <= uint.MaxValue) /* 4294967295 */
        {
            result = 4;
        }
        else if (integer <= ulong.MaxValue) /* 18446744073709551615 */
        {
            result = 8;
        }
        else
        {
            throw new ArgumentOutOfRangeException("Value is larger than GranularInteger.MaxValue.");
        }

        return result;
    }
}