﻿namespace EonaCat.Quic.Helpers;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

public enum StreamType
{
    ClientBidirectional = 0x0,
    ServerBidirectional = 0x1,
    ClientUnidirectional = 0x2,
    ServerUnidirectional = 0x3
}

public class StreamId
{
    public StreamId(ulong id, StreamType type)
    {
        Id = id;
        Type = type;
        IntegerValue = (id << 2) | (ulong)type;
    }

    public ulong Id { get; }
    public ulong IntegerValue { get; }
    public StreamType Type { get; }

    public static implicit operator byte[](StreamId id)
    {
        return Encode(id.Id, id.Type);
    }

    public static implicit operator StreamId(byte[] data)
    {
        return Decode(data);
    }

    public static implicit operator ulong(StreamId streamId)
    {
        return streamId.Id;
    }

    public static implicit operator StreamId(IntegerVar integer)
    {
        return Decode(ByteHelpers.GetBytes(integer.Value));
    }

    public static byte[] Encode(ulong id, StreamType type)
    {
        var identifier = (id << 2) | (ulong)type;

        var result = ByteHelpers.GetBytes(identifier);

        return result;
    }

    public static StreamId Decode(byte[] data)
    {
        StreamId result;
        var id = ByteHelpers.ToUInt64(data);
        var identifier = id >> 2;
        var type = 0x03 & id;
        var streamType = (StreamType)type;

        result = new StreamId(identifier, streamType);

        return result;
    }
}