﻿using System;

namespace EonaCat.Quic.Helpers;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

public class IntegerVar
{
    public const ulong MaxValue = 4611686018427387903;

    public IntegerVar(ulong integer)
    {
        Value = integer;
    }

    public ulong Value { get; }

    public static implicit operator byte[](IntegerVar integer)
    {
        return Encode(integer.Value);
    }

    public static implicit operator IntegerVar(byte[] bytes)
    {
        return new IntegerVar(Decode(bytes));
    }

    public static implicit operator IntegerVar(ulong integer)
    {
        return new IntegerVar(integer);
    }

    public static implicit operator ulong(IntegerVar integer)
    {
        return integer.Value;
    }

    public static implicit operator IntegerVar(StreamId streamId)
    {
        return new IntegerVar(streamId.IntegerValue);
    }

    public static int Size(byte firstByte)
    {
        var result = (int)Math.Pow(2, firstByte >> 6);

        return result;
    }

    public byte[] ToByteArray()
    {
        return Encode(Value);
    }

    public static byte[] Encode(ulong integer)
    {
        var requiredBytes = 0;
        if (integer <= byte.MaxValue >> 2) /* 63 */
        {
            requiredBytes = 1;
        }
        else if (integer <= ushort.MaxValue >> 2) /* 16383 */
        {
            requiredBytes = 2;
        }
        else if (integer <= uint.MaxValue >> 2) /* 1073741823 */
        {
            requiredBytes = 4;
        }
        else if (integer <= ulong.MaxValue >> 2) /* 4611686018427387903 */
        {
            requiredBytes = 8;
        }
        else
        {
            throw new ArgumentOutOfRangeException("Value is larger than IntegerVar.MaxValue.");
        }

        var offset = 8 - requiredBytes;

        var uInt64Bytes = ByteHelpers.GetBytes(integer);
        var first = uInt64Bytes[offset];
        first = (byte)(first | ((requiredBytes / 2) << 6));
        uInt64Bytes[offset] = first;

        var result = new byte[requiredBytes];
        Buffer.BlockCopy(uInt64Bytes, offset, result, 0, requiredBytes);

        return result;
    }

    public static ulong Decode(byte[] bytes)
    {
        var i = 8 - bytes.Length;
        var buffer = new byte[8];

        Buffer.BlockCopy(bytes, 0, buffer, i, bytes.Length);
        buffer[i] = (byte)(buffer[i] & (255 >> 2));

        var res = ByteHelpers.ToUInt64(buffer);

        return res;
    }
}