﻿using System.Collections.Generic;
using EonaCat.Quic.Helpers;

namespace EonaCat.Quic.Infrastructure.Packets;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

public class ShortHeaderPacket : Packet
{
    public byte ActualType = 0b0100_0000;

    public ShortHeaderPacket(byte destinationConnectionIdLength)
    {
        DestinationConnectionIdLength = destinationConnectionIdLength;
    }

    public override byte Type => 0b0100_0000;

    public IntegerParts DestinationConnectionId { get; set; }
    public IntegerParts PacketNumber { get; set; }

    // Field not transferred! Only the connection knows about the length of the ConnectionId
    public byte DestinationConnectionIdLength { get; set; }

    public override void Decode(byte[] packet)
    {
        var array = new ByteArray(packet);
        var type = array.ReadByte();
        DestinationConnectionId = array.ReadGranularInteger(DestinationConnectionIdLength);

        var pnSize = (type & 0x03) + 1;
        PacketNumber = array.ReadBytes(pnSize);

        DecodeFrames(array);
    }

    public override byte[] Encode()
    {
        var frames = EncodeFrames();

        var result = new List<byte>
        {
            (byte)(Type | (PacketNumber.Size - 1))
        };
        result.AddRange(DestinationConnectionId.ToByteArray());

        byte[] pnBytes = PacketNumber;
        result.AddRange(pnBytes);
        result.AddRange(frames);

        return result.ToArray();
    }
}