﻿using System;
using EonaCat.WebSockets.Buffer;

namespace EonaCat.WebSockets;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

public sealed class BinaryFragmentationFrame : Frame
{
    public BinaryFragmentationFrame(OpCode opCode, byte[] data, int offset, int count, bool isFin = false,
        bool isMasked = true)
    {
        BufferValidator.ValidateBuffer(data, offset, count, "data");

        OpCode = opCode;
        Data = data;
        Offset = offset;
        Count = count;
        IsFin = isFin;
        IsMasked = isMasked;
    }

    public byte[] Data { get; private set; }
    public int Offset { get; private set; }
    public int Count { get; private set; }
    public bool IsFin { get; private set; }
    public bool IsMasked { get; private set; }

    public override OpCode OpCode { get; }

    public byte[] ToArray(IFrameBuilder builder)
    {
        if (builder == null)
        {
            throw new ArgumentNullException("builder");
        }

        return builder.EncodeFrame(this);
    }
}