﻿using System;
using System.Linq;

namespace EonaCat.WebSockets;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

public class AsyncWebSocketRouteResolver
{
    private readonly AsyncWebSocketServerModuleCatalog _moduleCatalog;

    public AsyncWebSocketRouteResolver(AsyncWebSocketServerModuleCatalog moduleCatalog)
    {
        if (moduleCatalog == null)
        {
            throw new ArgumentNullException("moduleCatalog");
        }

        _moduleCatalog = moduleCatalog;
    }

    public AsyncWebSocketServerModule Resolve(string path, string query)
    {
        var modules = _moduleCatalog.GetAllModules();
        return modules.FirstOrDefault(m =>
            string.Compare(
                m.ModulePath.Trim().TrimStart('/').TrimEnd('/').ToLowerInvariant(),
                path.Trim().TrimStart('/').TrimEnd('/').ToLowerInvariant(),
                StringComparison.OrdinalIgnoreCase
            ) == 0);
    }
}