﻿using System;
using System.Collections.Generic;

namespace EonaCat.Network;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

/// <summary>
///     Request event arguments.
/// </summary>
public class RequestEventArgs : EventArgs
{
    internal RequestEventArgs(HttpContext context)
    {
        Ip = context.Request.Source.IpAddress;
        Port = context.Request.Source.Port;
        Method = context.Request.Method;
        Url = context.Request.Url.Full;
        Query = context.Request.Query.Elements;
        Headers = context.Request.Headers;
        ContentLength = context.Request.ContentLength;
    }

    /// <summary>
    ///     IP address.
    /// </summary>
    public string Ip { get; private set; }

    /// <summary>
    ///     Port number.
    /// </summary>
    public int Port { get; private set; }

    /// <summary>
    ///     HTTP method.
    /// </summary>
    public HttpMethod Method { get; private set; } = HttpMethod.GET;

    /// <summary>
    ///     URL.
    /// </summary>
    public string Url { get; private set; }

    /// <summary>
    ///     Query found in the URL.
    /// </summary>
    public Dictionary<string, string> Query { get; private set; } = new();

    /// <summary>
    ///     Request headers.
    /// </summary>
    public Dictionary<string, string> Headers { get; private set; } = new();

    /// <summary>
    ///     Content length.
    /// </summary>
    public long ContentLength { get; private set; }
}