﻿using System;
using System.Collections.Generic;

namespace EonaCat.Network;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

/// <summary>
///     Response event arguments.
/// </summary>
public class ResponseEventArgs : EventArgs
{
    internal ResponseEventArgs(HttpContext context, double totalMs)
    {
        Ip = context.Request.Source.IpAddress;
        Port = context.Request.Source.Port;
        Method = context.Request.Method;
        Url = context.Request.Url.Full;
        Query = context.Request.Query.Elements;
        RequestHeaders = context.Request.Headers;
        RequestContentLength = context.Request.ContentLength;
        StatusCode = context.Response.StatusCode;
        ResponseContentLength = context.Response.ContentLength;
        TotalMs = totalMs;
    }

    /// <summary>
    ///     IP address.
    /// </summary>
    public string Ip { get; private set; }

    /// <summary>
    ///     Port number.
    /// </summary>
    public int Port { get; private set; }

    /// <summary>
    ///     HTTP method.
    /// </summary>
    public HttpMethod Method { get; private set; } = HttpMethod.GET;

    /// <summary>
    ///     URL.
    /// </summary>
    public string Url { get; private set; }

    /// <summary>
    ///     Request query.
    /// </summary>
    public Dictionary<string, string> Query { get; private set; } = new();

    /// <summary>
    ///     Request headers.
    /// </summary>
    public Dictionary<string, string> RequestHeaders { get; private set; } = new();

    /// <summary>
    ///     Content length.
    /// </summary>
    public long RequestContentLength { get; private set; }

    /// <summary>
    ///     Response status.
    /// </summary>
    public int StatusCode { get; private set; }

    /// <summary>
    ///     Response headers.
    /// </summary>
    public Dictionary<string, string> ResponseHeaders { get; private set; } = new();

    /// <summary>
    ///     Response content length.
    /// </summary>
    public long? ResponseContentLength { get; private set; } = 0;

    /// <summary>
    ///     Total time in processing the request and sending the response, in milliseconds.
    /// </summary>
    public double TotalMs { get; private set; }
}