﻿using System;
using System.Net;
using EonaCat.Json;

namespace EonaCat.Network;
// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

/// <summary>
///     HTTP context including both request and response.
/// </summary>
public class HttpContext
{
    private readonly HttpListenerContext _Context;

    /// <summary>
    ///     Instantiate the object.
    /// </summary>
    public HttpContext()
    {
    }

    internal HttpContext(HttpListenerContext context, EonaCatWebserverSettings settings, EonaCatWebserverEvents events)
    {
        if (context == null)
        {
            throw new ArgumentNullException(nameof(context));
        }

        if (events == null)
        {
            throw new ArgumentNullException(nameof(events));
        }

        _Context = context;
        Request = new HttpRequest(context);
        Response = new HttpResponse(Request, context, settings, events);
    }

    /// <summary>
    ///     The HTTP request that was received.
    /// </summary>
    [JsonProperty(Order = -1)]
    public HttpRequest Request { get; }

    /// <summary>
    ///     Type of route.
    /// </summary>
    [JsonProperty(Order = 0)]
    public RouteTypeEnum? RouteType { get; internal set; } = null;

    /// <summary>
    ///     Matched route.
    /// </summary>
    [JsonProperty(Order = 1)]
    public object Route { get; internal set; } = null;

    /// <summary>
    ///     The HTTP response that will be sent.  This object is preconstructed on your behalf and can be modified directly.
    /// </summary>
    [JsonProperty(Order = 998)]
    public HttpResponse Response { get; private set; }

    /// <summary>
    ///     User-supplied metadata.
    /// </summary>
    [JsonProperty(Order = 999)]
    public object Metadata { get; set; } = null;
}